/* 

Copyright (C) 2003-2004 Yves Schmid (www.garagecube.com)

Author(s): Yves Schmid

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     GNU General Public License for more details.

You should have received a copy of the GNU General Public Licensealong with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#import "CosmicApp.h"
#import "CosmicPalette.h"
#import "CosmicMirror.h"
#import "CosmicAboutWindow.h"

#include "OMediaCanvas.h"

BOOL cosmicStopAllThread;
int  cosmitThreadsRunning;

@implementation CosmicApp

- (NSImage*)generateNSImageFromCanvas:(OMediaCanvas*)canvas
{
    NSImage		*img;
    NSBitmapImageRep	*myNSBitmapImageRep;
    NSRect 		bounds;
    int			dwidth = canvas->get_width(),
				dheight = canvas->get_height();

    if (!canvas) return NULL; 

    bounds = NSMakeRect (0, 0, dwidth, dheight);

    unsigned char	*pixels;
    unsigned char	*planes = NULL;
 
    canvas->lock(omlf_Read);

    pixels = (unsigned char*)canvas->get_pixels();
    
    myNSBitmapImageRep = [NSBitmapImageRep alloc];

    [myNSBitmapImageRep initWithBitmapDataPlanes:&planes
         pixelsWide:dwidth
         pixelsHigh:dheight
         bitsPerSample:8   
         samplesPerPixel:4 
         hasAlpha:YES
         isPlanar:NO 
         colorSpaceName:NSCalibratedRGBColorSpace
         bytesPerRow:dwidth*4
         bitsPerPixel:32];
    
    planes = [myNSBitmapImageRep bitmapData];
    
    memcpy(planes,pixels,dwidth*4*dheight);    

    img = [[NSImage alloc] init];
    [img addRepresentation:myNSBitmapImageRep];

    [img setSize:bounds.size];

    canvas->unlock();

    return img;
}


- (void)applicationDidFinishLaunching:(NSNotification *)notification 
{
	CosmicPalette   *palette = [CosmicPalette getMain];

	EnterMovies();

	[palette->panel setHidesOnDeactivate:NO];
    [palette->panel setBecomesKeyOnlyIfNeeded:YES];
	[palette->panel setLevel:NSNormalWindowLevel];
	[palette->panel makeKeyAndOrderFront:self];
//    [(NSPanel *)[self window] setFloatingPanel:YES];
	[aboutPanel makeKeyAndOrderFront:self];

	// Load image brushes

	for(int i=0;i<COSMIC_NIMAGES;i++)
	{
		NSData  *data;
	
		data = [[NSUserDefaults standardUserDefaults] objectForKey:[NSString stringWithFormat: @"Image_%d",i]];		
		if (data)
		{
			OMediaMemStream		stream;
			OMediaMoveableMem	*mb;
			long				size;

			size = [data length];
			stream.setsize(size);
			mb = stream.get_memory_block();

			memcpy(mb->lock(),[data bytes],size);
			mb->unlock();
			
			palette->imageCanvas[i] = new OMediaCanvas;
			stream>>palette->imageCanvas[i];

			palette->images[i] = [self generateNSImageFromCanvas:palette->imageCanvas[i]];
			if (i==0) [palette->imageView setImage:palette->images[i]];
		}
	}
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	int i;
	CosmicPalette   *palette = [CosmicPalette getMain];

	cosmicStopAllThread = YES;
	while(cosmitThreadsRunning>0) [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:1.0/20]];

	for(i=0;i<COSMIC_NIMAGES;i++)
	{
		if (palette->dirtyImages[i] && palette->imageCanvas[i])
		{
			OMediaMoveableMem	*mb;
			NSData				*data;
			long				size;
			OMediaMemStream		stream;

			stream.setposition(0);
			stream.setsize(0);

			palette->imageCanvas[i]->png_export(stream);

			mb = stream.get_memory_block();
			size = mb->getsize();
    
			data = [NSData dataWithBytes:mb->lock() length:size];
			mb->unlock();
			
			[[NSUserDefaults standardUserDefaults] setObject:data forKey:[NSString stringWithFormat: @"Image_%d",i]];		
		}
	}
	
}


- (BOOL)validateMenuItem:(NSMenuItem *)item 
{
    SEL action = [item action];

    if (action == @selector(showPalette:)) return YES;
        
    if (action == @selector(stopMirroring:)) 
	{
		if ([CosmicMirror getMirror]==NULL) return NO;
		return YES;	
	}

    return NO;
}

-(IBAction)stopMirroring:(id)sender
{
	[CosmicMirror stopMirroring];
}

-(IBAction)showPalette:(id)sender
{
	CosmicPalette   *palette = [CosmicPalette getMain];

	[palette->panel makeKeyAndOrderFront:self];
}


@end
